package model

// AdRelated 广告关联,每一条记录为一条当前用户来源关联的广告的详情
type AdRelated struct {
	Id         string `bson:"_id" json:"_id"`
	UserId     string `bson:"userId" json:"userId"`
	Aid        int64  `json:"aid" bson:"aid"` //广告的推广计划id,即广告ID，广告平台配置落地页参数
	Cid        string `json:"cid" bson:"cid"` //openid的用户点击aid广告进入游戏时的唯一标识,广告平台提供
	Pid        int64  `json:"pid" bson:"pid"` //广告的项目id(仅巨量引擎存在,腾讯广告时不存在该值),广告平台配置落地页参数
	CreateTime int64  `bson:"create_time"`    //当前计划的创建时间

	StartNum int     `bson:"startNum" json:"startNum"`   //启动次数
	Revenue  float32 `bson:"revenue" json:"revenue"`     //当日预估收益
	Duration int64   `bson:"duration" json:"duration"`   //当日在线时长
	ReqCount int     `bson:"req_count" json:"req_count"` //当日的激励视频广告请求次数
	ExpCount int     `bson:"exp_count" json:"exp_count"` //当日的激励视频广告曝光次数
}

type UserBehavior struct {
	Id                 string `bson:"_id,omitempty"`
	Gid                string `bson:"gid" json:"gid"`
	Pf                 string `bson:"pf" json:"pf"`
	OpenId             string `bson:"openId" json:"openId"`
	RelatedAid         int64  `bson:"relatedAid" json:"relatedAid"` //目前关联的aid
	TotalDuration      int    `bson:"totalDuration" json:"totalDuration"`
	TotalAdReqCount    int    `bson:"totalAdReqCount" json:"totalAdReqCount"`
	TotalAdEposedCount int    `bson:"totalAdEposedCount" json:"totalAdEposedCount"`
	CreateDate         string `bson:"createDate" json:"createDate"`
	CreateTime         int    `json:"createTime" bson:"createTime"`
	StartNum           int    `bson:"startNum" json:"startNum"`

	ActiveStatus     bool              `bson:"activeStatus" json:"activeStatus"`         //激活状态
	ConversionStatus bool              `bson:"conversionStatus" json:"conversionStatus"` //转化状态
	RemainData       map[string]string `json:"remainData" bson:"remainData"`             //留存数据
}

type UserBehaviorByDay struct {
	Id            string `bson:"_id,omitempty"`
	Date          string `bson:"date" json:"date"` //统计日期
	UserId        string `bson:"userId" json:"userId"`
	Duration      int    `bson:"duration" json:"duration"`
	AdReqCount    int    `bson:"adReqCount" json:"adReqCount"`
	AdEposedCount int    `bson:"adEposedCount" json:"adEposedCount"`
}

// ConversionCondition 转化条件
type ConversionCondition struct {
	Id               string  `bson:"_id" json:"id"`
	Gid              string  `bson:"gid" json:"gid"`
	Pid              int64   `bson:"pid" json:"pid"`
	Aid              int64   `bson:"aid" json:"aid"`
	Type             string  `bson:"type"  json:"type"`
	StartNum         int     `bson:"start_num" json:"start_num"` //启动次数
	EstimatedRevenue float32 `bson:"revenue" json:"revenue"`     //当日预估收益
	Duration         int64   `bson:"duration" json:"duration"`   //当日在线时长
	ReqRewardedAd    int     `bson:"req_count" json:"req_count"` //当日的激励视频广告请求次数
	ExpRewardedAd    int     `bson:"exp_count" json:"exp_count"` //当日的激励视频广告曝光次数
}
